@@ -4,6 +4,11 @@
  * See the file COPYRIGHT in the distribution package for copying terms.
  */
 
+#ifndef HAVE_STRUCT_ITIMERVAL
+#error no itimerval
+#define NOTIMER
+#endif
+
 #define WITH_RESOLV
 #undef DO_BIND
 
@@ -51,9 +56,15 @@
 #endif /* TIME_WITH_SYS_TIME */
 #include <errno.h>
 
+#ifdef HAVE_WINSOCK2_H
+#include <winsock2.h>
+#endif
+
 #include <fcntl.h>
 #include <sys/types.h>
+#ifdef HAVE_SYS_UIO_H
 #include <sys/uio.h>
+#endif
 #include <sys/stat.h>
 #if HAVE_SYS_WAIT_H || CRAENV
 #include <sys/wait.h>
@@ -90,14 +101,28 @@
 #endif /* either soundcard.h */
 #endif /* ndef NOSOUND */
 
+#ifndef HAVE_SYS_SOCKET_H
+#define NOTCP
+#endif
+
 #ifdef NOTCP
 #define NORESOLV 1
 #endif
 
 #ifndef NOTCP
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif
+
+#ifndef HAVE_NETDB_H
+#define NORESOLV
+#endif
 
 #ifndef NORESOLV
 #include <netdb.h>
@@ -144,9 +169,11 @@ struct progstate {
   int bytes_firsttransfer; /* How many bytes were transferred at the
 			    * first read/write pair? We need this for the
 			    * extended throughput calculation. */
+  long long bytes_lastreport; /* ... and since last report */
   double starttime; /* Time we began to try read/write */
   double lasttime; /* Last time we did a read/write */
   double time_firsttransfer; /* How much time did the first transfer took */
+  double time_lastreport; /* How much time since the last report */
   long long n_lines;
   int pidfile_has_been_created;
   int pid;
@@ -154,10 +181,12 @@ struct progstate {
 	      * the stream to.  Currently that will always be 1 (stdout) */
 };
 
+#ifndef NOTIMER 
 static void sigtimer(const int signal)
 {
   signal_report = 1;
 }
+#endif
 
 static void sigshutdown(const int signal)
 {
@@ -215,6 +244,7 @@ static void sighup(int signal)
   n_sighups++;
 }
 
+#ifdef SIGHUP
 static void handle_sighup(void)
 {
   fprintf(stderr,
@@ -231,6 +261,7 @@ static void handle_sighup(void)
   signal(SIGHUP, SIG_DFL);
   kill(getpid(), SIGHUP);
 }
+#endif
 
 static struct options *default_options(struct options *const o)
 {
@@ -238,6 +269,7 @@ static struct options *default_options(struct options *const o)
   return o;
 }
 
+#ifdef HAVE_STRUCT_SIGACTION
 static void tsignal(const int sig, void (*const handler)(int))
 {
   struct sigaction sa;
@@ -245,8 +277,20 @@ static void tsignal(const int sig, void (*const handler)(int))
   sa.sa_handler = handler;
   sigemptyset(&sa.sa_mask);
   sa.sa_flags = 0;
-  sigaction(sig, &sa, NULL);
+  if (sigaction(sig, &sa, NULL) == -1) {
+    perror("sigaction");
+    exit(2);
+  }
 }
+#else
+static void tsignal(const int sig, void (*const handler)(int))
+{
+  if (signal(sig, handler) == SIG_ERR) {
+    perror("signal");
+    exit(2);
+  }
+}
+#endif
 
 #ifdef WANT_SOUND
 
@@ -604,18 +648,27 @@ static void init(struct options *const o, struct progstate *const state)
   stopit = 0;
   signal_report = 0;
   n_sighups = 0;
+#ifdef SIGUSR2
   tsignal(SIGUSR2, sigshutdown);
+#endif
 
+#ifdef SIGUSR1
   tsignal(SIGUSR1, sigreport);
+#endif
 #ifdef SIGINFO
   if (!o->S)
     tsignal(SIGINFO, sigreport);
 #endif
+#ifdef SIGHUP
   tsignal(SIGHUP, sighup);
+#endif
 
   state->bytes_firsttransfer = -1;
   state->time_firsttransfer = -1.0;
 
+  state->bytes_lastreport = -1;
+  state->time_lastreport = -1.0;
+
   if (gettimeofday(&t, NULL) == -1) {
     perror("gettimeofday() failed");
     exit(2);
@@ -661,13 +714,15 @@ static void print_kmg(const char *const pre
 }
 
 static void report(const struct options *const o,
-		   const struct progstate *const state,
+		   struct progstate *const state,
 		   int curbytes)
 {
   struct timeval t2;
   double sofar;
   double rate;
+#ifndef NOTIMER
   struct itimerval itv;
+#endif
 
   if (gettimeofday(&t2, NULL) == -1) {
     perror("gettimeofday() failed");
@@ -706,14 +761,31 @@ static void report(const struct options *const o,
     fprintf(stderr,"Since end of first transfer: %.0f B/s", rate);
     print_kmg(" ", "%.2f", rate, "B/s", stderr);
     fprintf(stderr, "\n");
+
+    if (state->bytes_lastreport != -1) {
+      rate =
+	(
+	 (double)state->bytes_transferred - (double)state->bytes_lastreport
+	 ) / (sofar - state->time_lastreport);
+      fprintf(stderr,"Since last report          : %.0f B/s", rate);
+      print_kmg(" ", "%.2f", rate, "B/s", stderr);
+      fprintf(stderr, "\n");
+    }
   }
 
+  state->bytes_lastreport = state->bytes_transferred;
+  state->time_lastreport = sofar;
+
   if (o->T) {
+#ifndef NOTIMER
     itv.it_interval.tv_sec = o->T;
     itv.it_interval.tv_usec = 0;
     itv.it_value.tv_sec = o->T;
     itv.it_value.tv_usec = 0;
     setitimer(ITIMER_REAL, &itv, NULL);
+#else
+    fprintf(stderr, "WARNING: timer support not compiled in\n");
+#endif
   }
 }
 
@@ -958,11 +1030,15 @@ static void loop(struct options *const o, struct progstate *const state)
     }
   } else {
     /* In child process, don't answer to signals */
+#ifdef SIGUSR1
     signal(SIGUSR1, SIG_DFL);
+#endif
 #ifdef SIGINFO
     signal(SIGINFO, SIG_DFL);
 #endif
+#ifdef SIGHUP
     signal(SIGHUP, SIG_DFL);
+#endif
   }
 
 
@@ -1299,7 +1375,12 @@ int main(int argc, char *const argv[])
     case 'p': o.p = strdup(optarg); break;
     case 'V': print_version(); exit(0);
     case 'w': o.w = atoi_kmg(optarg); break;
-    case 'T': o.T = atoi(optarg); break;
+    case 'T': 
+#ifdef NOTIMER 
+      fprintf(stderr, "Warning: timer support not compiled in\n");
+#endif
+      o.T = atoi(optarg);
+      break;
     default: usage();
     }
   }
@@ -1321,6 +1402,7 @@ int main(int argc, char *const argv[])
   init(&o, &state);
 
   if (o.T) {
+#ifndef NOTIMER 
     struct itimerval itv;
 
     signal(SIGALRM, sigtimer);
@@ -1329,6 +1411,7 @@ int main(int argc, char *const argv[])
     itv.it_value.tv_sec = o.T;
     itv.it_value.tv_usec = 0;
     setitimer(ITIMER_REAL, &itv, NULL);
+#endif
   }
 
   if (o.c == 4) {
